@@ -1248,43 +1248,63 @@ static void write_handler(handler_t * handler, FILE * mimetypes_fp, FILE * cmds_
 }
 
 /**
- * Get the directory where processed config files are put
+ * Get the directory where cached config files are put
  *
  * @return allocated string containing the path
  */
-static char * get_processed_dir(void)
+static char * get_cache_dir(void)
 {
      char path[MAX_FILE_PATH_LEN];
-     const char * home = get_home_dir();
-     const char * alt = getenv("MOZPLUGGER_HOME");
-     int lenSoFar;
+     const char * home;
+     const char ** sub_dirs = NULL;
 
-     if(alt)
+     /* Locations are ...
+      * $MOZPLUGGER_HOME/.cache/
+      * $XDG_CACHE_HOME/mozplugger/
+      * $HOME/.cache/mozplugger/
+      */
+
+     if( (home = getenv("MOZPLUGGER_HOME")) != NULL)
      {
-          strncpy(path, alt, sizeof(path));
+          static const char * dirs[] = {".cache", 0};
+          sub_dirs = dirs;
      }
-     else if(home)
+     else if( (home = getenv("XDG_CACHE_HOME")) != NULL)
      {
-          snprintf(path, sizeof(path), "%s/.mozplugger", home);
+          static const char * dirs[] = {"mozplugger", 0};
+          sub_dirs = dirs;
+     }
+     else if( (home = get_home_dir()) != NULL)
+     {
+          static const char * dirs[] = {".cache", "mozplugger", 0};
+          sub_dirs = dirs;
      }
      else
      {
           ERROR("Neither MOZPLUGGER_HOME or HOME are set!\n");
      }
-
-     if((mkdir(path, 0777) != 0) && (errno != EEXIST))
+ 
+     strncpy(path, home, sizeof(path));
+    
+     do
      {
-          perror("Failed to create directory");
-          exit(EXIT_FAILURE);
-     }
+          int lenSoFar;
+          if((mkdir(path, 0777) != 0) && (errno != EEXIST))
+          {
+               perror("Failed to create directory");
+               exit(EXIT_FAILURE);
+          }
+          if( *sub_dirs == NULL)
+          {
+               break;
+          }
 
-     lenSoFar = strlen(path);
-     strncpy(&path[lenSoFar], "/.processed", sizeof(path) - lenSoFar);
-     if((mkdir(path, 0777) != 0) && (errno != EEXIST))
-     {
-          perror("Failed to create directory");
-          exit(EXIT_FAILURE);
+          lenSoFar = strlen(path);
+          snprintf(&path[lenSoFar], sizeof(path) - lenSoFar, "/%s", *sub_dirs);
+          sub_dirs++;
      }
+     while(1);
+
      return strdup(path);
 }
 
@@ -1313,7 +1333,7 @@ static size_t max_string_len(const char * str1, const char * str2)
  * @param[in] out filename of destination
  * @param[in] magic The magic string
  */
-static void copy(const char * in, const char * out, const char * magic)
+static int copy(const char * in, const char * out, const char * magic)
 {
     char buffer[BUF_LEN];
     FILE * inFp;
@@ -1326,7 +1346,8 @@ static void copy(const char * in, const char * out, const char * magic)
 
     if( !(inFp = fopen(in, "rb")))
     {
-        ERROR("Failed to open '%s'\n", in);
+        NOTICE("Failed to open '%s'\n", in);
+        return 0;
     }
     if( !(outFp = fopen(out, "wb")))
     {
@@ -1360,13 +1381,14 @@ static void copy(const char * in, const char * out, const char * magic)
         len += fread(&buffer[len], 1, BUF_LEN - len, inFp);
     }
 
-    fwrite(buffer, 1, len, outFp);
-    while((len = fread(buffer, 1, 1024, inFp)) > 0)
-    {
-        fwrite(buffer, 1, len, outFp);
-    }
-    fclose(inFp);
-    fclose(outFp);
+     fwrite(buffer, 1, len, outFp);
+     while((len = fread(buffer, 1, 1024, inFp)) > 0)
+     {
+          fwrite(buffer, 1, len, outFp);
+     }
+     fclose(inFp);
+     fclose(outFp);
+     return 1;
 }
 
 /**
@@ -1379,26 +1401,33 @@ static void copy(const char * in, const char * out, const char * magic)
  */
 static void install(const char * localPluginDir, int cfgIdx, int pluginIdx)
 {
-    char localPluginPath[MAX_FILE_PATH_LEN];
-    char tmpPluginPath[MAX_FILE_PATH_LEN];
-    char globalPluginPath[MAX_FILE_PATH_LEN];
-    char magic[MAX_PLUGIN_MAGIC_LEN];
-    char * sep;
+     char localPluginPath[MAX_FILE_PATH_LEN];
+     char tmpPluginPath[MAX_FILE_PATH_LEN];
+     char globalPluginPath[MAX_FILE_PATH_LEN];
+     char magic[MAX_PLUGIN_MAGIC_LEN];
+     char * p = GLOBAL_PLUGIN_DIRS;
 
-    snprintf(tmpPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger.tmp", localPluginDir);
-    snprintf(localPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger%i.so", localPluginDir, pluginIdx);
-    snprintf(magic, MAX_PLUGIN_MAGIC_LEN, "%i:", cfgIdx);
+     snprintf(tmpPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger.tmp", localPluginDir);
+     snprintf(localPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger%i.so", localPluginDir, pluginIdx);
+     snprintf(magic, MAX_PLUGIN_MAGIC_LEN, "%i:", cfgIdx);
 
-    unlink(tmpPluginPath);
-    rename(localPluginPath, tmpPluginPath);
+     unlink(tmpPluginPath);
+     rename(localPluginPath, tmpPluginPath);
 
-    strncpy(globalPluginPath, GLOBAL_PLUGIN_DIRS, sizeof(globalPluginPath));
-    if( (sep = strchr(globalPluginPath, ' ')) == NULL)
-    {
-        sep = &globalPluginPath[strlen(globalPluginPath)];
-    }
-    strncpy(sep, "/mozplugger.so", sizeof(globalPluginPath) - (sep - globalPluginPath));
-    copy(globalPluginPath, localPluginPath, magic);
+     do
+     {
+          char * q;
+          p = skip_spaces(p);
+          for(q = p; (*q != ' ') && (*q != '\t') && (*q != '\0'); q++)
+               ;
+          if(q == p)
+          {
+               ERROR("Failed to find 'mozplugger.so'\n");
+          }
+          snprintf(globalPluginPath, sizeof(globalPluginPath), "%.*s/mozplugger.so", q-p, p);
+          p = q;
+     }
+     while(!copy(globalPluginPath, localPluginPath, magic));
 }
 
 /*
@@ -1466,7 +1495,7 @@ static void removeRest(const char * localPluginDir, int idx)
 }
 
 /**
- * Write the plugin data to some post processed config files so that mozplugger
+ * Write the plugin data to some post cached config files so that mozplugger
  * can be quick.
  *
  * @param[in] localPluginDir The local(user) directory where plugins live.
@@ -1480,7 +1509,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
      FILE * fp2;
      handler_t * handler;
      char buffer[MAX_FILE_PATH_LEN];
-     char * path = get_processed_dir();
+     char * path = get_cache_dir();
 
      install(localPluginDir, cfgIdx, pluginIdx);
 
@@ -1531,7 +1560,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
  */
 static void write_ts_file(void)
 {
-     char * path = get_processed_dir();
+     char * path = get_cache_dir();
 
      if(path)
      {
@@ -1559,47 +1588,53 @@ int main(int argc, char * argv[])
      /* Places to search for the mozplugger config file */
      static const cfgPath_t mozillaCfgPaths[] =
      {
-          {"%s/%s",                  "MOZPLUGGER_HOME"},
-          {"%s/.mozplugger/%s",      "HOME"},
-          {"%s/.mozilla/%s",         "HOME"},
-          {"%s/%s",                  "MOZILLA_HOME"},
+          {"%s/%s",                    "MOZPLUGGER_HOME"},
+          {"%s/mozplugger/%s",         "XDG_CONFIG_HOME"},
+          {"%s/.config/mozplugger/%s", "HOME"},
+          {"%s/.mozplugger/%s",        "HOME"},
+          {"%s/.mozilla/%s",           "HOME"},
+          {"%s/%s",                    "MOZILLA_HOME"},
 #ifdef CONFDIR
-          {CONFDIR "/%s",            NULL},
+          {CONFDIR "/%s",              NULL},
 #endif
-          {"/etc/%s",                NULL},
-          {"/usr/etc/%s",            NULL},
-          {"/usr/local/mozilla/%s",  NULL},
-          {NULL,                     NULL}
+          {"/etc/%s",                  NULL},
+          {"/usr/etc/%s",              NULL},
+          {"/usr/local/mozilla/%s",    NULL},
+          {NULL,                       NULL}
      };
 
      /* Places to search for the mozplugger config file */
      static const cfgPath_t netscapeCfgPaths[] =
      {
-          {"%s/%s",                  "MOZPLUGGER_HOME"},
-          {"%s/.mozplugger/%s",      "HOME"},
-          {"%s/.netscape/%s",        "HOME"},
+          {"%s/%s",                    "MOZPLUGGER_HOME"},
+          {"%s/mozplugger/%s",         "XDG_CONFIG_HOME"},
+          {"%s/.config/mozplugger/%s", "HOME"},
+          {"%s/.mozplugger/%s",        "HOME"},
+          {"%s/.netscape/%s",          "HOME"},
 #ifdef CONFDIR
-          {CONFDIR "/%s",            NULL},
+          {CONFDIR "/%s",              NULL},
 #endif
-          {"/etc/%s",                NULL},
-          {"/usr/etc/%s",            NULL},
-          {"/usr/local/netscape/%s", NULL},
-          {NULL,                     NULL}
+          {"/etc/%s",                  NULL},
+          {"/usr/etc/%s",              NULL},
+          {"/usr/local/netscape/%s",   NULL},
+          {NULL,                       NULL}
      };
 
      /* Places to search for the mozplugger config file */
      static const cfgPath_t operaCfgPaths[] =
      {
-          {"%s/%s",                  "MOZPLUGGER_HOME"},
-          {"%s/.mozplugger/%s",      "HOME"},
-          {"%s/.opera/%s",           "HOME"},
-          {"%s/%s",                  "OPERA_DIR"},
+          {"%s/%s",                    "MOZPLUGGER_HOME"},
+          {"%s/mozplugger/%s",         "XDG_CONFIG_HOME"},
+          {"%s/.config/mozplugger/%s", "HOME"},
+          {"%s/.mozplugger/%s",        "HOME"},
+          {"%s/.opera/%s",             "HOME"},
+          {"%s/%s",                    "OPERA_DIR"},
 #ifdef CONFDIR
-          {CONFDIR "/%s",            NULL},
+          {CONFDIR "/%s",              NULL},
 #endif
-          {"/etc/%s",                NULL},
-          {"/usr/etc/%s",            NULL},
-          {NULL,                     NULL}
+          {"/etc/%s",                  NULL},
+          {"/usr/etc/%s",              NULL},
+          {NULL,                       NULL}
      };
 
      static const browser_t browsers[] =
@@ -1613,7 +1648,7 @@ int main(int argc, char * argv[])
      int i;
      int cfgIndex = 0;
 
-     /* first thing make sure processed directory exists */
+     /* first thing make sure cache directory exists */
      write_ts_file();
 
      if((argc > 1) && (strcmp(argv[1], "-v") == 0))
